@@ -15,7 +15,8 @@ sub printheader ($) {
 	if ($config{sslcookie}) {
 		print $session->header(-charset => 'utf-8',
 			-cookie => $session->cookie(-httponly => 1, -secure => 1));
-	} else {
+	}
+	else {
 		print $session->header(-charset => 'utf-8',
 			-cookie => $session->cookie(-httponly => 1));
 	}
@@ -40,7 +41,8 @@ sub showform ($$$$;@) {
 
 sub redirect ($$) {
 	my $q=shift;
-	my $url=shift;
+	eval q{use URI};
+	my $url=URI->new(shift);
 	if (! $config{w3mmode}) {
 		print $q->redirect($url);
 	}
@@ -51,7 +53,7 @@ sub redirect ($$) {
 }
 
 sub decode_cgi_utf8 ($) {
-	# decode_form_utf8 method is needed for 5.10
+	# decode_form_utf8 method is needed for 5.01
 	if ($] < 5.01) {
 		my $cgi = shift;
 		foreach my $f ($cgi->param) {
@@ -64,8 +66,9 @@ sub decode_form_utf8 ($) {
 	if ($] >= 5.01) {
 		my $form = shift;
 		foreach my $f ($form->field) {
+			my @value=map { decode_utf8($_) } $form->field($f);
 			$form->field(name  => $f,
-			             value => decode_utf8($form->field($f)),
+			             value => \@value,
 		                     force => 1,
 			);
 		}
@@ -143,7 +146,7 @@ sub cgi_postsignin ($$) {
 	}
 	else {
 		if ($config{sslcookie} && ! $q->https()) {
-			error(gettext("probable misconfiguration: sslcookie is set, but you are attepting to login via http, not https"));
+			error(gettext("probable misconfiguration: sslcookie is set, but you are attempting to login via http, not https"));
 		}
 		else {
 			error(gettext("login failed, perhaps you need to turn on cookies?"));
@@ -234,11 +237,15 @@ sub cgi_prefs ($$) {
 	showform($form, $buttons, $session, $q);
 }
 
-sub cgi_custom_failure ($$) {
-	my $header=shift;
+sub cgi_custom_failure ($$$) {
+	my $q=shift;
+	my $httpstatus=shift;
 	my $message=shift;
 
-	print $header;
+	print $q->header(
+		-status => $httpstatus,
+		-charset => 'utf-8',
+	);
 	print $message;
 
 	# Internet Explod^Hrer won't show custom 404 responses
@@ -252,16 +259,30 @@ sub check_banned ($$) {
 	my $q=shift;
 	my $session=shift;
 
+	my $banned=0;
 	my $name=$session->param("name");
-	if (defined $name) {
-		if (grep { $name eq $_ } @{$config{banned_users}}) {
-			$session->delete();
-			cgi_savesession($session);
-			cgi_custom_failure(
-				$q->header(-status => "403 Forbidden"),
-				gettext("You are banned."));
+	if (defined $name && 
+	    grep { $name eq $_ } @{$config{banned_users}}) {
+		$banned=1;
+	}
+
+	foreach my $b (@{$config{banned_users}}) {
+		if (pagespec_match("", $b,
+			ip => $ENV{REMOTE_ADDR},
+			name => defined $name ? $name : "",
+		)) {
+			$banned=1;
+			last;
 		}
 	}
+
+	if ($banned) {
+		$session->delete();
+		cgi_savesession($session);
+		cgi_custom_failure(
+			$q, "403 Forbidden",
+			gettext("You are banned."));
+	}
 }
 
 sub cgi_getsession ($) {
