@@ -14,6 +14,7 @@ my $no_chdir=0;
 sub import {
 	hook(type => "checkconfig", id => "git", call => \&checkconfig);
 	hook(type => "getsetup", id => "git", call => \&getsetup);
+	hook(type => "genwrapper", id => "git", call => \&genwrapper);
 	hook(type => "rcs", id => "rcs_update", call => \&rcs_update);
 	hook(type => "rcs", id => "rcs_prepedit", call => \&rcs_prepedit);
 	hook(type => "rcs", id => "rcs_commit", call => \&rcs_commit);
@@ -41,6 +42,7 @@ sub checkconfig () {
 			wrappermode => (defined $config{git_wrappermode} ? $config{git_wrappermode} : "06755"),
 		};
 	}
+
 	if (defined $config{git_test_receive_wrapper} &&
 	    length $config{git_test_receive_wrapper}) {
 		push @{$config{wrappers}}, {
@@ -49,6 +51,16 @@ sub checkconfig () {
 			wrappermode => (defined $config{git_wrappermode} ? $config{git_wrappermode} : "06755"),
 		};
 	}
+
+	# Avoid notes, parser does not handle and they only slow things down.
+	$ENV{GIT_NOTES_REF}="";
+	
+	# Run receive test only if being called by the wrapper, and not
+	# when generating same.
+	if ($config{test_receive} && ! exists $config{wrapper}) {
+		require IkiWiki::Receive;
+		IkiWiki::Receive::test();
+	}
 }
 
 sub getsetup () {
@@ -115,6 +127,16 @@ sub getsetup () {
 		},
 }
 
+sub genwrapper {
+	if ($config{test_receive}) {
+		require IkiWiki::Receive;
+		return IkiWiki::Receive::genwrapper();
+	}
+	else {
+		return "";
+	}
+}
+
 sub safe_git (&@) {
 	# Start a child process safely without resorting /bin/sh.
 	# Return command output or success state (in scalar context).
@@ -400,7 +422,10 @@ sub git_sha1 (;$) {
 		'--', $file);
 	if ($sha1) {
 		($sha1) = $sha1 =~ m/($sha1_pattern)/; # sha1 is untainted now
-	} else { debug("Empty sha1sum for '$file'.") }
+	}
+	else {
+		debug("Empty sha1sum for '$file'.");
+	}
 	return defined $sha1 ? $sha1 : q{};
 }
 
@@ -594,9 +619,15 @@ sub rcs_getctime ($) {
 	# Remove srcdir prefix
 	$file =~ s/^\Q$config{srcdir}\E\/?//;
 
-	my @sha1s = run_or_non('git', 'rev-list', 'HEAD', '--', $file);
-	my $ci    = git_commit_info($sha1s[$#sha1s], 1);
-	my $ctime = $ci->{'author_epoch'};
+	my @raw_lines = run_or_die('git', 'log', 
+		'--follow', '--no-merges',
+		'--pretty=raw', '--raw', '--abbrev=40', '--always', '-c',
+		'-r', '--', $file);
+	my @ci;
+	while (my $parsed = parse_diff_tree("", \@raw_lines)) {
+		push @ci, $parsed;
+	}
+	my $ctime = $ci[$#ci]->{'author_epoch'};
 	debug("ctime for '$file': ". localtime($ctime));
 
 	return $ctime;
