@@ -151,7 +151,7 @@ my %feedlinks;
 sub preprocess_inline (@) {
 	my %params=@_;
 	
-	if (! exists $params{pages}) {
+	if (! exists $params{pages} && ! exists $params{pagenames}) {
 		error gettext("missing pages parameter");
 	}
 	my $raw=yesno($params{raw});
@@ -159,7 +159,7 @@ sub preprocess_inline (@) {
 	my $rss=(($config{rss} || $config{allowrss}) && exists $params{rss}) ? yesno($params{rss}) : $config{rss};
 	my $atom=(($config{atom} || $config{allowatom}) && exists $params{atom}) ? yesno($params{atom}) : $config{atom};
 	my $quick=exists $params{quick} ? yesno($params{quick}) : 0;
-	my $feeds=exists $params{feeds} ? yesno($params{feeds}) : !$quick;
+	my $feeds=! $nested && (exists $params{feeds} ? yesno($params{feeds}) : !$quick);
 	my $emptyfeeds=exists $params{emptyfeeds} ? yesno($params{emptyfeeds}) : 1;
 	my $feedonly=yesno($params{feedonly});
 	if (! exists $params{show} && ! $archive) {
@@ -183,36 +183,50 @@ sub preprocess_inline (@) {
 		$params{template} = $archive ? "archivepage" : "inlinepage";
 	}
 
-	my @list=pagespec_match_list(
-		[ grep { $_ ne $params{page} } keys %pagesources ],
-		$params{pages}, location => $params{page});
+	my @list;
 
-	if (exists $params{sort} && $params{sort} eq 'title') {
-		@list=sort { pagetitle(basename($a)) cmp pagetitle(basename($b)) } @list;
-	}
-	elsif (exists $params{sort} && $params{sort} eq 'title_natural') {
-		eval q{use Sort::Naturally};
-		if ($@) {
-			error(gettext("Sort::Naturally needed for title_natural sort"));
+	if (exists $params{pagenames}) {
+		foreach my $p (qw(sort pages)) {
+			if (exists $params{$p}) {
+				error sprintf(gettext("the %s and %s parameters cannot be used together"),
+					"pagenames", $p);
+			}
+		}
+
+		@list = map { bestlink($params{page}, $_) }
+		        split ' ', $params{pagenames};
+
+		if (yesno($params{reverse})) {
+			@list=reverse(@list);
+		}
+
+		foreach my $p (@list) {
+			add_depends($params{page}, $p, deptype($quick ? "presence" : "content"));
 		}
-		@list=sort { Sort::Naturally::ncmp(pagetitle(basename($a)), pagetitle(basename($b))) } @list;
-	}
-	elsif (exists $params{sort} && $params{sort} eq 'mtime') {
-		@list=sort { $pagemtime{$b} <=> $pagemtime{$a} } @list;
-	}
-	elsif (! exists $params{sort} || $params{sort} eq 'age') {
-		@list=sort { $pagectime{$b} <=> $pagectime{$a} } @list;
 	}
 	else {
-		error sprintf(gettext("unknown sort type %s"), $params{sort});
-	}
+		my $num=0;
+		if ($params{show}) {
+			$num=$params{show};
+		}
+		if ($params{feedshow} && $num < $params{feedshow} && $num > 0) {
+			$num=$params{feedshow};
+		}
+		if ($params{skip} && $num) {
+			$num+=$params{skip};
+		}
 
-	if (yesno($params{reverse})) {
-		@list=reverse(@list);
+		@list = pagespec_match_list($params{page}, $params{pages},
+			deptype => deptype($quick ? "presence" : "content"),
+			filter => sub { $_[0] eq $params{page} },
+			sort => exists $params{sort} ? $params{sort} : "age",
+			reverse => yesno($params{reverse}),
+			($num ? (num => $num) : ()),
+		);
 	}
 
 	if (exists $params{skip}) {
-		@list=@list[$params{skip} .. scalar @list - 1];
+		@list=@list[$params{skip} .. $#list];
 	}
 	
 	my @feedlist;
@@ -230,14 +244,12 @@ sub preprocess_inline (@) {
 		@list=@list[0..$params{show} - 1];
 	}
 
-	add_depends($params{page}, $params{pages});
-	# Explicitly add all currently displayed pages as dependencies, so
-	# that if they are removed or otherwise changed, the inline will be
-	# sure to be updated.
-	add_depends($params{page}, join(" or ", $#list >= $#feedlist ? @list : @feedlist));
-	
 	if ($feeds && exists $params{feedpages}) {
-		@feedlist=grep { pagespec_match($_, $params{feedpages}, location => $params{page}) } @feedlist;
+		@feedlist = pagespec_match_list(
+			$params{page}, "($params{pages}) and ($params{feedpages})",
+			deptype => deptype($quick ? "presence" : "content"),
+			list => \@feedlist,
+		);
 	}
 
 	my ($feedbase, $feednum);
@@ -288,17 +300,7 @@ sub preprocess_inline (@) {
 		# Add a blog post form, with feed buttons.
 		my $formtemplate=template("blogpost.tmpl", blind_cache => 1);
 		$formtemplate->param(cgiurl => $config{cgiurl});
-		my $rootpage;
-		if (exists $params{rootpage}) {
-			$rootpage=bestlink($params{page}, $params{rootpage});
-			if (!length $rootpage) {
-				$rootpage=$params{rootpage};
-			}
-		}
-		else {
-			$rootpage=$params{page};
-		}
-		$formtemplate->param(rootpage => $rootpage);
+		$formtemplate->param(rootpage => rootpage(%params));
 		$formtemplate->param(rssurl => $rssurl) if $feeds && $rss;
 		$formtemplate->param(atomurl => $atomurl) if $feeds && $atom;
 		if (exists $params{postformtext}) {
@@ -330,12 +332,13 @@ sub preprocess_inline (@) {
 			error sprintf(gettext("nonexistant template %s"), $params{template});
 		}
 		my $template=HTML::Template->new(@params) unless $raw;
+		my $needcontent=$raw || (!($archive && $quick) && $template->query(name => 'content'));
 	
 		foreach my $page (@list) {
 			my $file = $pagesources{$page};
 			my $type = pagetype($file);
-			if (! $raw || ($raw && ! defined $type)) {
-				unless ($archive && $quick) {
+			if (! $raw) {
+				if ($needcontent) {
 					# Get the content before populating the
 					# template, since getting the content uses
 					# the same template if inlines are nested.
@@ -354,22 +357,24 @@ sub preprocess_inline (@) {
 					my $file = $pagesources{$page};
 					my $type = pagetype($file);
 					if ($config{discussion}) {
-						my $discussionlink=gettext("discussion");
-						if ($page !~ /.*\/\Q$discussionlink\E$/ &&
+						if ($page !~ /.*\/\Q$config{discussionpage}\E$/i &&
 						    (length $config{cgiurl} ||
-						     exists $links{$page."/".$discussionlink})) {
+						     exists $pagesources{$page."/".lc($config{discussionpage})})) {
 							$template->param(have_actions => 1);
 							$template->param(discussionlink =>
 								htmllink($page,
 									$params{destpage},
-									gettext("Discussion"),
+									$config{discussionpage},
 									noimageinline => 1,
 									forcesubpage => 1));
 						}
 					}
-					if (length $config{cgiurl} && defined $type) {
+					if (length $config{cgiurl} &&
+					    defined $type &&
+					    IkiWiki->can("cgi_editpage")) {
 						$template->param(have_actions => 1);
 						$template->param(editurl => cgiurl(do => "edit", page => $page));
+
 					}
 				}
 	
@@ -389,6 +394,10 @@ sub preprocess_inline (@) {
 					      filter($page, $params{destpage},
 					      readfile(srcfile($file)))));
 				}
+				else {
+					$ret.="\n".
+					      readfile(srcfile($file));
+				}
 			}
 		}
 	}
@@ -417,6 +426,8 @@ sub preprocess_inline (@) {
 		}
 	}
 	
+	clear_inline_content_cache();
+
 	return $ret if $raw || $nested;
 	push @inline, $ret;
 	return "<div class=\"inline\" id=\"$#inline\"></div>\n\n";
@@ -431,25 +442,42 @@ sub pagetemplate_inline (@) {
 		if exists $feedlinks{$page} && $template->query(name => "feedlinks");
 }
 
+{
+my %inline_content;
+my $cached_destpage="";
+
 sub get_inline_content ($$) {
 	my $page=shift;
 	my $destpage=shift;
 	
+	if (exists $inline_content{$page} && $cached_destpage eq $destpage) {
+		return $inline_content{$page};
+	}
+
 	my $file=$pagesources{$page};
 	my $type=pagetype($file);
+	my $ret="";
 	if (defined $type) {
 		$nested++;
-		my $ret=htmlize($page, $destpage, $type,
+		$ret=htmlize($page, $destpage, $type,
 		       linkify($page, $destpage,
 		       preprocess($page, $destpage,
 		       filter($page, $destpage,
 		       readfile(srcfile($file))))));
 		$nested--;
-		return $ret;
 	}
-	else {
-		return "";
+	
+	if ($cached_destpage ne $destpage) {
+		clear_inline_content_cache();
+		$cached_destpage=$destpage;
 	}
+	return $inline_content{$page}=$ret;
+}
+
+sub clear_inline_content_cache () {
+	%inline_content=();
+}
+
 }
 
 sub date_822 ($) {
@@ -636,4 +664,21 @@ sub pingurl (@) {
 	exit 0; # daemon done
 }
 
+
+sub rootpage (@) {
+	my %params=@_;
+
+	my $rootpage;
+	if (exists $params{rootpage}) {
+		$rootpage=bestlink($params{page}, $params{rootpage});
+		if (!length $rootpage) {
+			$rootpage=$params{rootpage};
+		}
+	}
+	else {
+		$rootpage=$params{page};
+	}
+	return $rootpage;
+}
+
 1
