@@ -88,7 +88,7 @@ sub preprocess (@) {
 	# Metadata collection that needs to happen during the scan pass.
 	if ($key eq 'title') {
 		$pagestate{$page}{meta}{title}=HTML::Entities::encode_numeric($value);
-		# fallthrough
+		return "";
 	}
 	elsif ($key eq 'description') {
 		$pagestate{$page}{meta}{description}=HTML::Entities::encode_numeric($value);
@@ -121,6 +121,10 @@ sub preprocess (@) {
 		$pagestate{$page}{meta}{authorurl}=$value if safeurl($value);
 		# fallthrough
 	}
+	elsif ($key eq 'permalink') {
+		$pagestate{$page}{meta}{permalink}=$value if safeurl($value);
+		# fallthrough
+	}
 	elsif ($key eq 'date') {
 		eval q{use Date::Parse};
 		if (! $@) {
@@ -141,10 +145,9 @@ sub preprocess (@) {
 		return;
 	}
 
-	# Metadata collection that happens only during preprocessing pass.
+	# Metadata handling that happens only during preprocessing pass.
 	if ($key eq 'permalink') {
 		if (safeurl($value)) {
-			$pagestate{$page}{meta}{permalink}=$value;
 			push @{$metaheaders{$page}}, scrub('<link rel="bookmark" href="'.encode_entities($value).'" />', $destpage);
 		}
 	}
@@ -163,17 +166,22 @@ sub preprocess (@) {
 			"\" type=\"text/css\" />";
 	}
 	elsif ($key eq 'openid') {
+		my $delegate=0; # both by default
+		if (exists $params{delegate}) {
+			$delegate = 1 if lc $params{delegate} eq 'openid';
+			$delegate = 2 if lc $params{delegate} eq 'openid2';
+		}
 		if (exists $params{server} && safeurl($params{server})) {
 			push @{$metaheaders{$page}}, '<link href="'.encode_entities($params{server}).
-				'" rel="openid.server" />';
+				'" rel="openid.server" />' if $delegate ne 2;
 			push @{$metaheaders{$page}}, '<link href="'.encode_entities($params{server}).
-				'" rel="openid2.provider" />';
+				'" rel="openid2.provider" />' if $delegate ne 1;
 		}
 		if (safeurl($value)) {
 			push @{$metaheaders{$page}}, '<link href="'.encode_entities($value).
-				'" rel="openid.delegate" />';
+				'" rel="openid.delegate" />' if $delegate ne 2;
 			push @{$metaheaders{$page}}, '<link href="'.encode_entities($value).
-				'" rel="openid2.local_id" />';
+				'" rel="openid2.local_id" />' if $delegate ne 1;
 		}
 		if (exists $params{"xrds-location"} && safeurl($params{"xrds-location"})) {
 			push @{$metaheaders{$page}}, '<meta http-equiv="X-XRDS-Location"'.
@@ -186,11 +194,11 @@ sub preprocess (@) {
 		if ($value !~ /^\w+:\/\//) {
 			my ($redir_page, $redir_anchor) = split /\#/, $value;
 
-			add_depends($page, $redir_page);
 			my $link=bestlink($page, $redir_page);
 			if (! length $link) {
 				error gettext("redir page not found")
 			}
+			add_depends($page, $link, deptype("presence"));
 
 			$value=urlto($link, $page);
 			$value.='#'.$redir_anchor if defined $redir_anchor;
@@ -231,6 +239,10 @@ sub preprocess (@) {
 		push @{$metaheaders{$page}}, '<meta name="robots"'.
 			' content="'.encode_entities($value).'" />';
 	}
+	elsif ($key eq 'description') {
+		push @{$metaheaders{$page}}, '<meta name="'.encode_entities($key).
+			'" content="'.encode_entities($value).'" />';
+	}
 	else {
 		push @{$metaheaders{$page}}, scrub('<meta name="'.encode_entities($key).
 			'" content="'.encode_entities($value).'" />', $destpage);
@@ -255,7 +267,7 @@ sub pagetemplate (@) {
 		$template->param(title_overridden => 1);
 	}
 
-	foreach my $field (qw{author authorurl permalink}) {
+	foreach my $field (qw{author authorurl description permalink}) {
 		$template->param($field => $pagestate{$page}{meta}{$field})
 			if exists $pagestate{$page}{meta}{$field} && $template->query(name => $field);
 	}
@@ -286,21 +298,21 @@ sub match {
 
 	if (defined $val) {
 		if ($val=~/^$re$/i) {
-			return IkiWiki::SuccessReason->new("$re matches $field of $page");
+			return IkiWiki::SuccessReason->new("$re matches $field of $page", $page => $IkiWiki::DEPEND_CONTENT, "" => 1);
 		}
 		else {
-			return IkiWiki::FailReason->new("$re does not match $field of $page");
+			return IkiWiki::FailReason->new("$re does not match $field of $page", "" => 1);
 		}
 	}
 	else {
-		return IkiWiki::FailReason->new("$page does not have a $field");
+		return IkiWiki::FailReason->new("$page does not have a $field", "" => 1);
 	}
 }
 
 package IkiWiki::PageSpec;
 
 sub match_title ($$;@) {
-	IkiWiki::Plugin::meta::match("title", @_);	
+	IkiWiki::Plugin::meta::match("title", @_);
 }
 
 sub match_author ($$;@) {
