@@ -23,8 +23,7 @@ sub canedit ($$) {
 	my $cgi=shift;
 	my $session=shift;
 
-	my $discussion=gettext("discussion");
-	return "" if $page=~/(\/|^)\Q$discussion\E$/;
+	return "" if $page=~/(\/|^)\Q$config{discussionpage}\E$/i;
 	return undef;
 }
 
