@@ -35,19 +35,44 @@ sub preprocess (@) {
 	$params{pages}="*" unless defined $params{pages};
 	my $style = ($params{style} or 'cloud');
 	
-	# Needs to update whenever a page is added or removed, so
-	# register a dependency.
-	add_depends($params{page}, $params{pages});
-	
 	my %counts;
 	my $max = 0;
-	foreach my $page (pagespec_match_list([keys %links],
-			$params{pages}, location => $params{page})) {
+	foreach my $page (pagespec_match_list($params{page}, $params{pages},
+		                  # update when a displayed page is added/removed
+	        	          deptype => deptype("presence"))) {
 		use IkiWiki::Render;
-		$counts{$page} = scalar(IkiWiki::backlinks($page));
+
+		my @backlinks = IkiWiki::backlink_pages($page);
+
+		if (exists $params{among}) {
+			# only consider backlinks from the amoung pages
+			@backlinks = pagespec_match_list(
+				$params{page}, $params{among},
+				# update whenever links on those pages change
+				deptype => deptype("links"),
+				list => \@backlinks
+			);
+		}
+		else {
+			# update when any page with links changes,
+			# in case the links point to our displayed pages
+			add_depends($params{page}, "*", deptype("links"));
+		}
+
+		$counts{$page} = scalar(@backlinks);
 		$max = $counts{$page} if $counts{$page} > $max;
 	}
 
+	if (exists $params{show}) {
+		my $i=0;
+		my %show;
+		foreach my $key (sort { $counts{$b} <=> $counts{$a} } keys %counts) {
+			last if ++$i > $params{show};
+			$show{$key}=$counts{$key};
+		}
+		%counts=%show;
+	}
+
 	if ($style eq 'table') {
 		return "<table class='pageStats'>\n".
 			join("\n", map {
@@ -63,6 +88,8 @@ sub preprocess (@) {
 
 		my $res = "<div class='pagecloud'>\n";
 		foreach my $page (sort keys %counts) {
+			next unless $counts{$page} > 0;
+
 			my $class = $classes[$counts{$page} * scalar(@classes) / ($max + 1)];
 			$res .= "<span class=\"$class\">".
 			        htmllink($params{page}, $params{destpage}, $page).
