@@ -7,14 +7,23 @@ package IkiWiki::Plugin::polygen;
 
 use warnings;
 use strict;
-use IkiWiki 2.00;
+use IkiWiki 3.00;
 use File::Find;
 
-sub import { #{{{
+sub import {
+	hook(type => "getsetup", id => "polygen", call => \&getsetup);
 	hook(type => "preprocess", id => "polygen", call => \&preprocess);
-} # }}}
+}
 
-sub preprocess (@) { #{{{
+sub getsetup () {
+	return 
+		plugin => {
+			safe => 1,
+			rebuild => undef,
+		},
+}
+
+sub preprocess (@) {
 	my %params=@_;
 	my $grammar = ($params{grammar} or 'polygen');
 	my $symbol = ($params{symbol} or undef);
@@ -55,6 +64,6 @@ sub preprocess (@) { #{{{
 	# markdown text
 	$res =~ s/\s*$//;
 	return $res;
-} # }}}
+}
 
 1
