@@ -10,7 +10,7 @@ sub import {
 	hook(type => "formbuilder_setup", id => "rename", call => \&formbuilder_setup);
 	hook(type => "formbuilder", id => "rename", call => \&formbuilder);
 	hook(type => "sessioncgi", id => "rename", call => \&sessioncgi);
-
+	hook(type => "rename", id => "rename", call => \&rename_subpages);
 }
 
 sub getsetup () {
@@ -87,6 +87,27 @@ sub check_canrename ($$$$$$) {
 			IkiWiki::Plugin::attachment::check_canattach($session, $dest, $srcfile);
 		}
 	}
+
+	my $canrename;
+	IkiWiki::run_hooks(canrename => sub {
+		return if defined $canrename;
+		my $ret=shift->(cgi => $q, session => $session,
+			src => $src, srcfile => $srcfile,
+			dest => $dest, destfile => $destfile);
+		if (defined $ret) {
+			if ($ret eq "") {
+				$canrename=1;
+			}
+			elsif (ref $ret eq 'CODE') {
+				$ret->();
+				$canrename=0;
+			}
+			elsif (defined $ret) {
+				error($ret);
+				$canrename=0;
+			}
+		}
+	});
 }
 
 sub rename_form ($$$) {
@@ -214,6 +235,7 @@ sub formbuilder (@) {
 
 	if (defined $form->field("do") && ($form->field("do") eq "edit" ||
 	    $form->field("do") eq "create")) {
+    		IkiWiki::decode_form_utf8($form);
 		my $q=$params{cgi};
 		my $session=$params{session};
 
@@ -221,7 +243,7 @@ sub formbuilder (@) {
 			rename_start($q, $session, 0, $form->field("page"));
 		}
 		elsif ($form->submitted eq "Rename Attachment") {
-			my @selected=$q->param("attachment_select");
+			my @selected=map { Encode::decode_utf8($_) } $q->param("attachment_select");
 			if (@selected > 1) {
 				error(gettext("Only one attachment can be renamed at a time."));
 			}
@@ -257,7 +279,7 @@ sub sessioncgi ($$) {
 
 	if ($q->param("do") eq 'rename') {
         	my $session=shift;
-		my ($form, $buttons)=rename_form($q, $session, $q->param("page"));
+		my ($form, $buttons)=rename_form($q, $session, Encode::decode_utf8($q->param("page")));
 		IkiWiki::decode_form_utf8($form);
 
 		if ($form->submitted eq 'Cancel') {
@@ -269,9 +291,9 @@ sub sessioncgi ($$) {
 
 			# These untaints are safe because of the checks
 			# performed in check_canrename later.
-			my $src=$q->param("page");
+			my $src=$form->field("page");
 			my $srcfile=IkiWiki::possibly_foolish_untaint($pagesources{$src});
-			my $dest=IkiWiki::possibly_foolish_untaint(titlepage($q->param("new_name")));
+			my $dest=IkiWiki::possibly_foolish_untaint(titlepage($form->field("new_name")));
 			my $destfile=$dest;
 			if (! $q->param("attachment")) {
 				my $type=$q->param('type');
@@ -293,27 +315,13 @@ sub sessioncgi ($$) {
 				required => 1,
 			};
 
-			# See if any subpages need to be renamed.
-			if ($q->param("subpages") && $src ne $dest) {
-				foreach my $p (keys %pagesources) {
-					next unless $pagesources{$p}=~m/^\Q$src\E\//;
-					# If indexpages is enabled, the
-					# srcfile should not be confused
-					# with a subpage.
-					next if $pagesources{$p} eq $srcfile;
+			@torename=rename_hook(
+				torename => \@torename,
+				done => {},
+				cgi => $q,
+				session => $session,
+			);
 
-					my $d=$pagesources{$p};
-					$d=~s/^\Q$src\E\//$dest\//;
-					push @torename, {
-						src => $p,
-						srcfile => $pagesources{$p},
-						dest => pagename($d),
-						destfile => $d,
-						required => 0,
-					};
-				}
-			}
-			
 			require IkiWiki::Render;
 			IkiWiki::disable_commit_hook() if $config{rcs};
 			my %origpagesources=%pagesources;
@@ -407,7 +415,40 @@ sub sessioncgi ($$) {
 		exit 0;
 	}
 }
-						
+
+# Add subpages to the list of pages to be renamed, if needed.
+sub rename_subpages (@) {
+	my %params = @_;
+
+	my %torename = %{$params{torename}};
+	my $q = $params{cgi};
+	my $src = $torename{src};
+	my $srcfile = $torename{src};
+	my $dest = $torename{dest};
+	my $destfile = $torename{dest};
+
+	return () unless ($q->param("subpages") && $src ne $dest);
+
+	my @ret;
+	foreach my $p (keys %pagesources) {
+		next unless $pagesources{$p}=~m/^\Q$src\E\//;
+		# If indexpages is enabled, the srcfile should not be confused
+		# with a subpage.
+		next if $pagesources{$p} eq $srcfile;
+
+		my $d=$pagesources{$p};
+		$d=~s/^\Q$src\E\//$dest\//;
+		push @ret, {
+			src => $p,
+			srcfile => $pagesources{$p},
+			dest => pagename($d),
+			destfile => $d,
+			required => 0,
+		};
+	}
+	return @ret;
+}
+
 sub linklist {
 	# generates a list of links in a form suitable for FormBuilder
 	my $dest=shift;
@@ -439,7 +480,43 @@ sub renamepage_hook ($$$$) {
 
 	return $content;
 }
-			
+
+sub rename_hook {
+	my %params = @_;
+
+	my @torename=@{$params{torename}};
+	my %done=%{$params{done}};
+	my $q=$params{cgi};
+	my $session=$params{session};
+
+	return () unless @torename;
+
+	my @nextset;
+	foreach my $torename (@torename) {
+		unless (exists $done{$torename->{src}} && $done{$torename->{src}}) {
+			IkiWiki::run_hooks(rename => sub {
+				push @nextset, shift->(
+					torename => $torename,
+					cgi => $q,
+					session => $session,
+				);
+			});
+			$done{$torename->{src}}=1;
+		}
+	}
+
+	push @torename, rename_hook(
+		torename => \@nextset,
+		done => \%done,
+		cgi => $q,
+		session => $session,
+	);
+
+	# dedup
+	my %seen;
+	return grep { ! $seen{$_->{src}}++ } @torename;
+}
+
 sub do_rename ($$$) {
 	my $rename=shift;
 	my $q=shift;
