@@ -21,19 +21,30 @@ sub getsetup () {
 		},
 		add_underlays => {
 			type => "string",
-			default => [],
+			example => ["$ENV{HOME}/wiki.underlay"],
 			description => "extra underlay directories to add",
 			advanced => 1,
 			safe => 0,
 			rebuild => 1,
 		},
+		add_templates => {
+			type => "string",
+			example => ["$ENV{HOME}/.ikiwiki/templates"],
+			description => "extra template directories to add",
+			advanced => 1,
+			safe => 0,
+			rebuild => 1,
+		},
 }
 
 sub checkconfig () {
-	return unless exists $config{add_underlays};
-
-	foreach my $dir (@{$config{add_underlays}}) {
-		add_underlay($dir);
+	if ($config{add_underlays}) {
+		foreach my $dir (@{$config{add_underlays}}) {
+			add_underlay($dir);
+		}
+	}
+	if ($config{add_templates}) {
+		push @{$config{templatedirs}}, @{$config{add_templates}};
 	}
 }
 
