@@ -24,16 +24,21 @@ sub prettydir ($) {
 	return $dir;
 }
 
+sub sanitize_wikiname ($) {
+	my $wikiname=shift;
+
+	# Sanitize this to avoid problimatic directory names.
+	$wikiname=~s/[^-A-Za-z0-9_]//g;
+	if (! length $wikiname) {
+		error gettext("you must enter a wikiname (that contains alphanumerics)");
+	}
+	return $wikiname;
+}
+
 sub import (@) {
 	my $this=shift;
 	IkiWiki::Setup::merge({@_});
 
-	# Sanitize this to avoid problimatic directory names.
-	$config{wikiname}=~s/[^-A-Za-z0-9_]//g;
-	if (! length $config{wikiname}) {
-		error gettext("you must enter a wikiname (that contains alphanumerics)");
-	}
-
 	# Avoid overwriting any existing files.
 	foreach my $key (qw{srcdir destdir repository dumpsetup}) {
 		next unless exists $config{$key};
@@ -67,6 +72,9 @@ sub import (@) {
 		elsif ($config{rcs} eq 'mercurial') {
 			# TODO
 		}
+		elsif ($config{rcs} eq 'cvs') {
+			$config{cvs_wrapper}=$config{repository}."/CVSROOT/post-commit";
+		}
 		else {
 			error sprintf(gettext("unsupported revision control system %s"),
 			       	$config{rcs});
@@ -98,6 +106,29 @@ sub import (@) {
 		}
 	}
 
+	# Make sure that all the listed plugins can load
+	# and checkconfig is ok. If a plugin fails to work,
+	# remove it from the configuration and keep on truckin'.
+	my %bakconfig=%config; # checkconfig can modify %config so back up
+	if (! eval { IkiWiki::loadplugins(); IkiWiki::checkconfig() }) {
+		foreach my $plugin (@{$config{default_plugins}}, @{$bakconfig{add_plugins}}) {
+			eval {
+				# delete all hooks so that only this plugins's
+				# checkconfig will be run
+				%IkiWiki::hooks=();
+				IkiWiki::loadplugin($plugin);
+				IkiWiki::run_hooks(checkconfig => sub { shift->() });
+			};
+			if ($@) {
+				print STDERR sprintf(gettext("** Disabling plugin %s, since it is failing with this message:"),
+					$plugin)."\n";
+				print STDERR "$@\n";
+				push @{$bakconfig{disable_plugins}}, $plugin;
+			}
+		}
+	}
+	%config=%bakconfig;
+
 	# Generate setup file.
 	require IkiWiki::Setup;
 	IkiWiki::Setup::dump($config{dumpsetup});
